//+------------------------------------------------------------------+
//|                                                    Functions.mqh |
//|                                    2019-2021, dimitri pecheritsa |
//|                                         mql5.com/en/users/dmipec |
//|------------------------------------------------------------------|
//| fnc | miscellaneous                                              |
//|------------------------------------------------------------------|
//| use | gnome sort example                                         |
//| 010 | symbols load (symbols[])                                   |
//| 020 | symbols keys color (symbols[i], keys[o])                   |
//| 030 | symbols print (symbols[])                                  |
//| 040 | array sort (keys[i/o], items[i/o], algorithm, order)       |
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
//| 010 | symbols load                                               |
//|------------------------------------------------------------------|
//| ret | number of copied symbols                                   |
//| out | symbols | destination array for the market watch symbols   |
//+------------------------------------------------------------------+
int SymbolsLoad(string& dst_symbols[])
  {
   int total=SymbolsTotal(true);
   for(int i=0; i<total; i++)
     {
      int size=ArraySize(dst_symbols);
      ArrayResize(dst_symbols,size+1,size*2);
      dst_symbols[size]=SymbolName(i,true);
     }
   return total;
  }
//+------------------------------------------------------------------+
//| 020 | symbols keys spread                                        |
//|------------------------------------------------------------------|
//| use | fill keys array by extracting symbol spread from the source|
//|symbols array for each of its element                             |
//| in  | symbols | source array with symbols                        |
//| out | keys    | destination array                                |
//+------------------------------------------------------------------+
void SymbolKeysSpread(string& symbols[],int& spread[])
  {
   int total=ArraySize(symbols);
   ArrayResize(spread,total);
   for(int i=0; i<total; i++)
      spread[i]=(int)SymbolInfoInteger(symbols[i],SYMBOL_SPREAD);
  }
//+------------------------------------------------------------------+
//| 030 | symbols print                                              |
//|------------------------------------------------------------------|
//| use | print a table of symbols consiting of three columns: name, |
//|spread, point                                                     |
//| in  | symbols | source array to print                            |
//+------------------------------------------------------------------+
void SymbolsPrint(string& symbols[])
  {
//--- format table columns
   string format="%21s | %8s | %8s";
//--- create horizontal table line
   string line;
   StringInit(line,68,'-');
//--- header
   Print(line);
   printf(format,"symbol","spread","point");
   Print(line);
//--- body
   int total=ArraySize(symbols);
   for(int i=0; i<total; i++)
     {
      //--- data
      int spread=(int)SymbolInfoInteger(symbols[i],SYMBOL_SPREAD);
      double point=SymbolInfoDouble(symbols[i],SYMBOL_POINT);
      //--- out
      printf(format,symbols[i],(string)spread,StringFormat("%G",point));
     }
//--- footer
   Print(line);
  }
//+------------------------------------------------------------------+
//| 040 | array sort                                                 |
//|------------------------------------------------------------------|
//| use | sorts items by keys with the specified concrete sorting    |
//|algorithm, in accending or descending order                       |
//| ret | true if sorted; false if error                             |
//| i/o | keys      | array which values are compared and sorted     |
//| i/o | items     | array which items are swapped, and thus sorted |
//|the same way as the keys array. items are ignored if its length   |
//|does not match the length of the keys                             |
//| in  | algorithm | any compatible sorting algorithm               |
//| in  | order     | sorting order flag                             |
//+------------------------------------------------------------------+
#include <Mqh\Algorithms\ASorter.mqh>
template<typename TKey,typename TItem>
bool ArraySort(TKey& keys[],TItem& items[],ASorter<TKey,TItem>* algorithm,ENUM_SORT_ORDER order=SORT_ORDER_ACCENDING)
  {
//--- check
   if(!CheckPointer(algorithm))
      return false;
//--- prepare algorithm
   ArrayCopy(algorithm.keys,keys);
   ArrayCopy(algorithm.items,items);
//--- sort keys and items
   algorithm.Sort();
//--- save sorted items
   ArrayCopy(keys,algorithm.keys);
   ArrayCopy(items,algorithm.items);
//--- keep order
   if(order==SORT_ORDER_DESCENDING)
     {
      ArrayReverse(keys);
      ArrayReverse(items);
     }
//--- clear
   if(CheckPointer(algorithm)==1)
      delete algorithm;
//--- exit
   return true;
  }
//+------------------------------------------------------------------+
